/*
 * buttons.c - button api
 *
 * author: hackbard@hackdaworld.org
 *
 */

#include "buttons.h"

#define BUTTON_RESET \
	IOSET2=((1<<18)|(1<<19)|(1<<20)|(1<<21)|(1<<22)|(1<<23)|(1<<24))

#define BUTTON_UNSET \
	IOCLR2=((1<<18)|(1<<19)|(1<<20)|(1<<21)|(1<<22)|(1<<23)|(1<<24))

#define COUNT_AND_CHECK 	cnt++; \
				if(cnt==BUTTON_MAX) \
					break

void button_init(t_button *button) {

	/*
	 * input:
	 *  - p0.30 : interrupt! wtf is the 'vw 3 p9'? a transistor?
	 *  - p0.28, p0.27, p3.21, p3.20, p0.22, p0.13 : the columns
	 *
	 * output:
	 *  - p2.18-p2.24 : the rows
	 */

	/* poll / interrupt mode */
	if(button->mode&BUTTON_INT)
		BUTTON_UNSET;
	else if(button->mode&BUTTON_POLL)
		BUTTON_RESET;
}

void button_set_retries(t_button *button,int retries) {

	button->retries=retries;
}

void button_select_row(u8 row) {

	BUTTON_RESET;
	if(row>6)
		return;
	IOCLR2=(1<<(18+row));
}

u8 button_get_event(t_button *button) {

	u8 row;
	u8 offset;
	u8 cnt;
	u32 port0,port3;
	u8 tries;

	cnt=0;
	tries=button->retries+1;

	while(tries--) {
		/* rest offset counter */
		offset=0;
		/* rows */
		for(row=0;row<7;row++) {
			/* select the row */
			button_select_row(row);
			/* scan the columns 6 */
			port0=IOPIN0;
			port3=IOPIN3;
			if(!(port0&(1<<28))) {
				button->key[cnt]=offset+0;
				COUNT_AND_CHECK;
			}
			if(!(port0&(1<<27))) {
				button->key[cnt]=offset+1;
				COUNT_AND_CHECK;
			}
			if(!(port0&(1<<22))) {
				button->key[cnt]=offset+2;
				COUNT_AND_CHECK;
			}
			if(!(port0&(1<<13))) {
				button->key[cnt]=offset+3;
				COUNT_AND_CHECK;
			}
			if(!(port3&(1<<21))) {
				button->key[cnt]=offset+4;
				COUNT_AND_CHECK;
			}
			if(!(port3&(1<<20))) {
				button->key[cnt]=offset+5;
				COUNT_AND_CHECK;
			}
			offset+=6;
		}
		if(cnt)
			break;
	}

	if(button->mode&BUTTON_INT)
		BUTTON_UNSET;
	else if(button->mode&BUTTON_POLL)
		BUTTON_RESET;

	return cnt;
}

